import { HttpClient, HttpParams } from "@angular/common/http";
import { Injectable } from "@angular/core";
import { BackendError } from "app/shared/models/backend-error";
import { BaseService } from "app/shared/services/base.service";
import { environment } from "environments/environment";
import { Observable } from "rxjs";
import { catchError, map } from "rxjs/operators";
import { ParticipantAttendance } from "../dto/participant-attendance.dto";
import { CreateAttendanceDto } from "../dto/create-attendance.dto";
import { Attendance } from "../models/attendance";

@Injectable({
    providedIn: 'root'
})
export class AttendanceService extends BaseService {
    constructor(private http: HttpClient) {
        super();
    }

    createAttendance(dto: CreateAttendanceDto): Observable<Attendance> {
        return this.http.put<Attendance>(`${environment.backendUrl}/attendance`, dto)
            .pipe(
                catchError(e => {
                    throw BackendError.parse(e);
                }),
            );
    }

    deleteAttendance(userId: number, eventId: number): Observable<void> {
        return this.http.delete<void>(`${environment.backendUrl}/attendance/user/${userId}/event/${eventId}`)
            .pipe(
                catchError(e => {
                    throw BackendError.parse(e);
                }),
            );
    }
}