import { Component, Input, OnInit } from '@angular/core';
import { User } from 'app/core/models/user';
import { DistinctHelper } from 'app/shared/helper/distinct.helper';
import { Course } from '../../models/course';
import { StaffGroup } from '../../models/staff-group';

interface StaffMember {
  user: User;
  groups: StaffGroup[];
}

@Component({
  selector: 'ngx-course-staff',
  templateUrl: './course-staff.component.html',
  styleUrls: ['./course-staff.component.scss']
})
export class CourseStaffComponent implements OnInit {
  @Input() course: Course;
  @Input() compactMode: boolean;
  
  members: StaffMember[];
  StaffGroup = StaffGroup;

  constructor() { }

  ngOnInit(): void {
    this.computeMembers();
  }

  computeMembers() {
    let users = DistinctHelper.distinct(
      this.course.admins.concat(this.course.lecturers).concat(this.course.tutors), 
      u => u.id
    );

    this.members = users.map(u => ({
      user: u,
      groups: this.getGroups(u)
    }));
  }

  getGroups(user: User) {
    let groups: StaffGroup[] = [];

    if(this.isAdmin(user)) {
      groups.push(StaffGroup.Admin);
    }
    if(this.isLecturer(user)) {
      groups.push(StaffGroup.Lecturer);
    }
    if(this.isTutor(user)) {
      groups.push(StaffGroup.Tutor);
    }

    return groups;
  }

  isAdmin(user: User) {
    return this.course.admins.map(u => u.id).indexOf(user.id) != -1;
  }

  isTutor(user: User) {
    return this.course.tutors.map(u => u.id).indexOf(user.id) != -1;
  }

  isLecturer(user: User) {
    return this.course.lecturers.map(u => u.id).indexOf(user.id) != -1;
  }
}
