import { HttpClient } from '@angular/common/http';
import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { Course } from '../../models/course';
import { CourseService } from '../../services/course.service';
import { takeUntil, flatMap, debounceTime, filter } from 'rxjs/operators';
import { Subject } from 'rxjs';
import { DataQueryParams } from 'app/shared/models/data-query-params';

@Component({
  selector: 'ngx-course-template-list',
  templateUrl: './course-template-list.component.html',
  styleUrls: ['./course-template-list.component.scss']
})
export class CourseTemplateListComponent extends BaseComponent implements OnInit {
  loading: boolean;
  selectedCourse: Course | null = null;
  courses: Course[] = [];
  @Output() courseSelected: EventEmitter<Course> = new EventEmitter<Course>();

  courseQuery: string;
  private courseQueryChanged: Subject<string> = new Subject<string>();

  courseTemplateQueryParams: DataQueryParams = { sortBy: 'term.id', sortOrder: 'desc' };

  constructor(
    private http: HttpClient,
    private courseService: CourseService
  ) {
    super();
  }

  ngOnInit(): void {
    this.loading = true;
    this.courseService.getAll(null, this.courseTemplateQueryParams)
    .pipe(takeUntil(this.unsubscribe))
    .subscribe(courses => {
      this.courses = courses;
      this.loading = false;
    });

    this.courseQueryChanged
      .pipe(
        takeUntil(this.unsubscribe),
        debounceTime(500),
        flatMap(query => this.courseService.getAll({ query: query }, this.courseTemplateQueryParams))
      )
      .subscribe(courses => {
        this.courses = courses;
      });
  }

  selectCourse(course: Course): void {
    this.selectedCourse = course;
    this.courseSelected.emit(this.selectedCourse);
  }

  filterCourses(query) {
    this.courseQueryChanged.next(query);
  }
}
