import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { AuthService } from 'app/core/services/auth.service';
import { NotificationService } from 'app/shared/services/notification.service';
import { combineLatest, forkJoin } from 'rxjs';
import { CourseUnitEvent } from '../../models/course-unit-event';
import { takeUntil, flatMap } from 'rxjs/operators';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { CourseUnitService } from '../../services/course-unit.service';
import { CourseUnit } from '../../models/course-unit';
import { FormArray, UntypedFormBuilder, FormGroup, Validators } from '@angular/forms';
import { User } from 'app/core/models/user';
import { BackendError } from 'app/shared/models/backend-error';
import { ErrorContainerConfig } from 'app/shared/components/error-container/error-container.component';
import { AppStateService } from 'app/core/services/app.state.service';
import { Breadcrumb } from 'app/shared/components/breadcrumbs/breadcrumbs.component';
import { SubatoPrincipal } from 'app/core/models/principal/subato-principal';
import { ParamHelper } from 'app/shared/helper/param.helper';

@Component({
  selector: 'ngx-course-unit-admin',
  templateUrl: './course-unit-admin.component.html',
  styleUrls: ['./course-unit-admin.component.scss']
})
export class CourseUnitAdminComponent extends BaseComponent implements OnInit {
  loading: boolean;
  unit: CourseUnit;
  principal: SubatoPrincipal;
  participants: User[] = [];
  events: CourseUnitEvent[] = [];
  error: BackendError;
  errorConfig: ErrorContainerConfig = {
    notFound: {
      message: 'Diese Veranstaltung konnten wir leider nicht finden.'
    }
  };

  breadcrumbs: Breadcrumb[] = [];

  constructor(
    private courseUnitService: CourseUnitService,
    private appStateService: AppStateService,
    private notificationService: NotificationService,
    private route: ActivatedRoute,
    private authService: AuthService,
    private router: Router,
    private fb: UntypedFormBuilder
  ) {
    super();
  }

  buildBreadcrumbs(unit: CourseUnit) {
    this.breadcrumbs = [
      {
        id: 'courseUnits',
        params: {
          course_id: unit.course.id
        },
      },
      {
        id: 'courseUnit',
        params: {
          unit_id: unit.id
        },
        ctx: unit
      },
      {
        id: 'courseUnitAdmin',
        params: {
          unit_id: unit.id
        },
        ctx: unit
      }
    ];
  }

  ngOnInit(): void {
    this.loading = true;
    combineLatest(this.route.paramMap, this.authService.principal$)
      .pipe(
        takeUntil(this.unsubscribe),
        flatMap(([params, principal]) => {
          this.principal = principal;
          let unitId = ParamHelper.parseIdOrThrow(params, "unit_id");

          return this.courseUnitService.getById(unitId);
        })
      )
      .subscribe(unit => {
        this.unit = unit;
        this.appStateService.setCourse(unit.course);
        this.buildBreadcrumbs(unit);
        this.loading = false;
      }, error => {
        if (BackendError.isBackendError(error)) {
          this.error = error;
          this.loading = false;
        } else {
          throw error;
        }
      });
  }

}
