import { Component, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { AppStateService } from 'app/core/services/app.state.service';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { Breadcrumb } from 'app/shared/components/breadcrumbs/breadcrumbs.component';
import { ErrorContainerConfig } from 'app/shared/components/error-container/error-container.component';
import { ParamHelper } from 'app/shared/helper/param.helper';
import { BackendError } from 'app/shared/models/backend-error';
import { takeUntil, flatMap } from 'rxjs/operators';
import { Exercise } from '../../models/exercise';
import { ExerciseService } from '../../services/exercise.service';

@Component({
  selector: 'ngx-edit-exercise',
  templateUrl: './edit-exercise.component.html',
  styleUrls: ['./edit-exercise.component.scss']
})
export class EditExerciseComponent extends BaseComponent implements OnInit {
  loading: boolean = true;
  exercise: Exercise;

  error: BackendError;
  errorConfig: ErrorContainerConfig = {
    notFound: {
      message: 'Diese Übung konnten wir leider nicht finden.'
    }
  };

  breadcrumbs: Breadcrumb[] = [];

  constructor(
    private exerciseService: ExerciseService,
    private appStateService: AppStateService,
    private route: ActivatedRoute) {
    super();
  }

  buildBreadcrumbs(exercise: Exercise) {
    this.breadcrumbs = [
      {
        id: 'exercises',
        params: {
          course_id: exercise.course.id
        },
      },
      {
        id: 'exercise',
        params: {
          exercise_id: exercise.id
        },
        ctx: exercise
      },
      {
        id: 'editExercise',
        params: {
          exercise_id: exercise.id
        },
        ctx: exercise
      },
    ];
  }

  ngOnInit(): void {
    this.route.paramMap
      .pipe(
        takeUntil(this.unsubscribe),
        flatMap(params => {
          let exerciseId = ParamHelper.parseIdOrThrow(params, "exercise_id");
          return this.exerciseService.getById(exerciseId);
        }),
      ).subscribe(exercise => {
        this.exercise = exercise;
        this.appStateService.setCourse(exercise.course);
        this.buildBreadcrumbs(exercise);

        this.loading = false;
      }, (error: any) => {
        if (BackendError.isBackendError(error)) {
          this.error = error;
          this.loading = false;
        } else {
          throw error;
        }
      });
  }
}
