import {  AfterViewInit, Component, Input, OnInit, QueryList, ViewChild, ViewChildren } from '@angular/core';
import { CodemirrorComponent } from '@ctrl/ngx-codemirror';
import { AppStateService } from 'app/core/services/app.state.service';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { CodemirrorHelper } from 'app/shared/helper/codemirror.helper';
import { take, takeUntil } from 'rxjs/operators';
import { EvalResult, EvalService, EvalStatus } from '../../services/eval.service';


@Component({
  selector: 'ngx-eval',
  templateUrl: './eval.component.html',
  styleUrls: ['./eval.component.css']
})
export class EvalComponent extends BaseComponent implements OnInit {
  @Input() code: string = "";
  @Input() highlight: string | null = null;
  result: EvalResult | null = null;
  
  isProcessing = false;
  selectedTabIndex: number | null = null;

  darkMode = false;

  constructor(
    private appStateService: AppStateService,
    private evalService: EvalService,
  ) { 
    super();
  }

  ngOnInit(): void {
    this.appStateService.darkMode$
    .pipe(takeUntil(this.unsubscribe))
    .subscribe(darkMode => this.darkMode = darkMode);
  }


  get accent() {
    if(!this.result) {
      return 'basic';
    }

    return this.isSuccess ? 'primary' : 'danger';
  }

  execute(code: string) {
    this.isProcessing = true;
    this.evalService.eval({ code: code })
      .pipe(takeUntil(this.unsubscribe))
      .subscribe(res => {
        res.out = res.out?.trim();
        this.result = res;

        if(res.status == EvalStatus.cerror) {
          this.selectedTabIndex = 2;
        } else {
          if(!res.out && res.result) {
            this.selectedTabIndex = 1;
          } else {
            this.selectedTabIndex = 0;
          }
        }
  
        this.isProcessing = false;
      });
  }

  get hasCompilerErrors() { return this.result && this.result.status == EvalStatus.cerror; }

  get isError() { return this.result && this.result.status != EvalStatus.ok; }
  get isSuccess() { return this.result && this.result.status == EvalStatus.ok; }

  formatErrors(result: EvalResult) {
      var r = 'Fehler bei der Kompilierung:'; 
      result.errors.forEach(e => {
        r += '\n' + e.message;
        if(e.snippet && (!e.startPos || !e.endPos)) {
          r += ': ' + e.snippet;
        }
      });

    return r;
  }

  
}
