import { Component, OnInit } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { Breadcrumb } from 'app/shared/components/breadcrumbs/breadcrumbs.component';
import { ErrorContainerConfig } from 'app/shared/components/error-container/error-container.component';
import { ParamHelper } from 'app/shared/helper/param.helper';
import { BackendError } from 'app/shared/models/backend-error';
import { Observable, of } from 'rxjs';
import { takeUntil, flatMap } from 'rxjs/operators';
import { TaskPool } from '../../models/task-pool';
import { TaskPoolService } from '../../services/task-pool.service';

@Component({
  selector: 'ngx-edit-task-pool',
  templateUrl: './edit-task-pool.component.html',
  styleUrls: ['./edit-task-pool.component.scss']
})
export class EditTaskPoolComponent extends BaseComponent implements OnInit {
  loading: boolean = true;
  pool: TaskPool;

  error: BackendError;
  errorConfig: ErrorContainerConfig = {
    notFound: {
      message: 'Diesen Aufgaben-Pool konnten wir leider nicht finden.'
    }
  };

  breadcrumbs: Breadcrumb[] = [];

  constructor(
    private poolService: TaskPoolService,
    private router: Router,
    private route: ActivatedRoute) {
    super();
  }


  buildBreadcrumbs(pool: TaskPool) {
    this.breadcrumbs = [
      {
        id: 'pools',
      },
      {
        id: 'pool',
        params: {
          pool_id: pool.id
        },
        ctx: pool
      },
      {
        id: 'editPool',
        params: {
          pool_id: pool.id
        },
      }
    ];
  }

  ngOnInit(): void {
    this.route.paramMap
      .pipe(
        takeUntil(this.unsubscribe),
        flatMap(params => {
          let poolId = params.has("pool_id") ? ParamHelper.parseIdOrThrow(params, "pool_id") : null;

          let loadPool: Observable<TaskPool> = of(null);
          if (poolId != null) {
            loadPool = this.poolService.getById(poolId);
          }

          return loadPool;
        })
      ).subscribe(pool => {
        this.pool = pool;
        this.buildBreadcrumbs(pool);
        this.loading = false;
      }, error => {
        if (BackendError.isBackendError(error)) {
          this.error = error;
          this.loading = false;
        } else {
          throw error;
        }
      });
  }

}
