import { Directive, Inject } from "@angular/core";
import { take } from "rxjs/operators";
import { DOCUMENT } from '@angular/common';
import { environment } from "environments/environment";
import { DownloadService } from "../services/download.service";
import { BackendError } from "../models/backend-error";
import { NotificationService } from "../services/notification.service";

@Directive({
  selector: "[downloadLinkIntercept]",
})
export class DownloadLinkInterceptDirective {
  constructor(
    @Inject(DOCUMENT) private document: Document,
    private notificationService: NotificationService,
    private downloadService: DownloadService) {

    this.document.body.addEventListener('click', ev => {
      var isLink = (<Element>ev.target).matches('a');

      if (isLink) {
        var link = <HTMLAnchorElement>ev.target;
        this.onLinkClicked(ev, link);
      }
    })
  }

  onLinkClicked(event: any, link: HTMLAnchorElement) {
    let hasDownloadAttribute = link.attributes.getNamedItem("download") != null;
    if (hasDownloadAttribute) {
        let href = link.attributes.getNamedItem("href").value;
        this.downloadService.downloadFile(href)
          .pipe(
            take(1),
          ).subscribe(() => {}, error => {
            if (BackendError.isBackendError(error)) {
              this.notificationService.showError(
                error.format(
                  "Fehler beim Herunterladen der Datei",
                )
              );
            } else {
              throw error;
            }
          });
        event.preventDefault();
    }
  }
}