package de.hsrm.sls.subato.attendances.attendance;

import de.hsrm.sls.subato.attendances.attendance.model.Attendance;
import de.hsrm.sls.subato.attendances.attendance.model.ParticipantAttendance;
import de.hsrm.sls.subato.courses.course.Course;
import de.hsrm.sls.subato.courses.course.CourseService;
import de.hsrm.sls.subato.courses.event.CourseUnitEvent;
import de.hsrm.sls.subato.courses.event.CourseUnitEventService;
import de.hsrm.sls.subato.courses.unit.CourseUnit;
import de.hsrm.sls.subato.courses.unit.CourseUnitService;
import de.hsrm.sls.subato.courses.unit.CourseUnitType;
import de.hsrm.sls.subato.user.User;
import de.hsrm.sls.subato.user.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
@Transactional
public class AttendanceService {
    @Autowired
    private AttendanceRepository attendanceRepository;

    @Autowired
    private UserService userService;

    @Autowired
    private CourseUnitEventService courseUnitEventService;

    @Autowired
    private CourseUnitService courseUnitService;

    @Autowired
    private CourseService courseService;

    public List<ParticipantAttendance> getAttendanceForCourseAndUser(long courseId, long userId) {
        var course = courseService.findById(courseId);
        var user = userService.findById(userId);
        return getAttendanceForCourseAndUser(course, user);
    }

    public List<ParticipantAttendance> getAttendanceForCourseAndUser(Course course, User user) {
        var courseUnits = user.getParticipatingUnits().stream()
                .filter(unit -> unit.getCourse().getId() == course.getId())
                .filter(unit -> unit.getType() == CourseUnitType.Lab);

        return courseUnits.map(unit -> getAttendanceForUnitAndUser(unit, user)).toList();
    }

    public  ParticipantAttendance getAttendanceForUnitAndUser(CourseUnit courseUnit, User user) {
        var attendances = attendanceRepository.findByStudentAndCourseUnit(user.getId(), courseUnit.getId());

        var unitAttendance = new ParticipantAttendance(user, courseUnit);
        for(var event : courseUnit.getEvents()) {
            var attendance = attendances.stream().filter(
                    a -> a.getStudent().getId() == user.getId() && a.getEvent().getId() == event.getId()
            ).findFirst().orElse(null);

            if(attendance != null) {
                unitAttendance.getEventAttendances().add(attendance);
            }
        }

        return unitAttendance;
    }

    public void deleteAttendance(User user, CourseUnitEvent event) {
        var attendance = attendanceRepository.findByStudentIdAndEventId(user.getId(), event.getId()).orElseThrow(() -> new AttendanceNotFoundException(user.getId(), event.getId()));
        attendanceRepository.delete(attendance);
    }

    public Attendance createAttendance(User user, CourseUnitEvent event) {
        var attendance = attendanceRepository.findByStudentIdAndEventId(user.getId(), event.getId()).orElse(null);

        if(attendance == null) {
            attendance = new Attendance();
            attendance.setStudent(user);
            attendance.setEvent(event);
            attendance = attendanceRepository.save(attendance);
        }

        return attendance;
    }
}
