package de.hsrm.sls.subato.courses.file;

import de.hsrm.sls.subato.shared.config.SubatoConfig;
import de.hsrm.sls.subato.shared.fs.FileContent;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.FileSystemUtils;

import java.io.File;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Arrays;

@Service
public class CourseFileContentRepository {
    @Autowired
    private SubatoConfig subatoConfig;

    public Path getCourseFilePath(CourseFile courseFile) {
        return Paths.get(subatoConfig.getDataDir())
                .resolve("courseFile")
                .resolve(String.valueOf(courseFile.getId()));
    }

    public void store(FileContent content, CourseFile courseFile) {
        var dir = getCourseFilePath(courseFile).toFile();
        content.saveTo(dir);
    }

    public File getFile(CourseFile courseFile) {
        var path = getCourseFilePath(courseFile);
        var dir = path.toFile();
        var firstFile = Arrays.stream(dir.listFiles()).findFirst().orElse(null);
        if(firstFile.isDirectory()) {
            // this should actually never happen
            // because store() stores exactly one file in the dir (which is not unzipped if its a zip file)
            throw new RuntimeException("File for courseFile %s at path %s is a directory?".formatted(courseFile.getId(), firstFile.toPath()));
        }
        return firstFile;
    }

    public void delete(CourseFile courseFile) {
        var courseFilePath = getCourseFilePath(courseFile);
        FileSystemUtils.deleteRecursively(courseFilePath.toFile());
    }
}
