package de.hsrm.sls.subato.seed.exercise;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import de.hsrm.sls.subato.courses.course.CourseRepository;
import de.hsrm.sls.subato.exercises.exercise.Exercise;
import de.hsrm.sls.subato.exercises.exercise.ExerciseRepository;
import de.hsrm.sls.subato.exercises.taskinstance.TaskInstance;
import de.hsrm.sls.subato.seed.SeedingException;
import de.hsrm.sls.subato.tasks.task.TaskRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.util.List;

@Service
@Transactional
public class ExerciseSeedService {
    private static Logger logger = LoggerFactory.getLogger(ExerciseSeedService.class);

    @Autowired
    ExerciseRepository exerciseRepository;

    @Autowired
    CourseRepository courseRepository;

    @Autowired
    TaskRepository taskRepository;

    public void seedExercises(File seedDir, ObjectMapper objectMapper) {
        try {
            var termsPath = seedDir.toPath().resolve("exercises.json");
            var json = Files.readString(termsPath, StandardCharsets.UTF_8);
            var exerciseSeeds = objectMapper.readValue(json,  new TypeReference<List<ExerciseSeed>>() {});

            logger.debug("Seeding %s exercises...".formatted(exerciseSeeds.size()));

            for(var exerciseSeed : exerciseSeeds) {
                if(exerciseSeed.getId() != null && exerciseRepository.existsBySourceId(exerciseSeed.getId())) {
                    continue;
                }

                var exercise = new Exercise();
                exercise.setSourceId(exerciseSeed.getId());
                exercise.setName(exerciseSeed.getName());
                exercise.setDescription(exerciseSeed.getDescription());
                exercise.setDueTo(exerciseSeed.getDueTo());
                exercise.setAssessmentMode(exerciseSeed.getAssessmentMode());
                exercise.setSummary(exerciseSeed.getSummary());

                var course = courseRepository.findBySourceId(exerciseSeed.getCourseId()).orElse(null);
                exercise.setCourse(course);

                for(var taskInstanceSeed : exerciseSeed.getTaskInstances()) {
                    var taskInstance = new TaskInstance();
                    var task = taskRepository.findBySourceId(taskInstanceSeed.getTaskId()).orElse(null);
                    if(task == null) {
                        continue;
                    }

                    taskInstance.setTask(task);
                    taskInstance.setExercise(exercise);
                    taskInstance.setSolutionPublic(taskInstanceSeed.isSolutionPublic());
                    taskInstance.setOrderNumber(taskInstanceSeed.getOrderNumber());
                    exercise.getTaskInstances().add(taskInstance);
                }

                exerciseRepository.save(exercise);
            }
        } catch (IOException e) {
            throw new SeedingException(e);
        }
    }
}
