package de.hsrm.sls.subato.shared;

import de.hsrm.sls.subato.shared.mime.MimeDetector;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.Resource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;

import java.io.IOException;

@Component
public class FileResponseBuilder {
    @Autowired
    MimeDetector mimeDetector;

    public ResponseEntity build(Resource resource, boolean asAttachment) throws IOException {
        return build(resource, resource.getFilename(), asAttachment);
    }

    public ResponseEntity build(Resource resource, String fileName, boolean asAttachment) throws IOException {
        String contentType = mimeDetector.getContentType(fileName);
        var disposition = asAttachment ? "attachment" : "inline";

        return ResponseEntity.ok().contentType(MediaType.parseMediaType(contentType))
                .header(HttpHeaders.CONTENT_DISPOSITION, disposition + "; filename=\"" + fileName + "\"")
                .body(resource);
    }
}
