package de.hsrm.sls.subato.tasks.pool.sync;

import de.hsrm.sls.subato.utils.SubatoTmpDirCleanup;
import de.hsrm.sls.subato.Tags;
import de.hsrm.sls.subato.tasks.pool.TaskPoolRepository;
import de.hsrm.sls.subato.tasks.pool.sync.repository.GitConfig;
import de.hsrm.sls.subato.tasks.pool.sync.repository.TaskPoolRepositoryManager;
import org.junit.jupiter.api.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.jdbc.AutoConfigureTestDatabase;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.transaction.annotation.Transactional;

/**
 * Integration test for managing git repositories (task pools)
 *
 * Remote dependencies:
 * - GitLab (access to repository of task pool #1)
 */
@SpringBootTest
@DirtiesContext(classMode = DirtiesContext.ClassMode.BEFORE_EACH_TEST_METHOD)
@AutoConfigureTestDatabase(replace = AutoConfigureTestDatabase.Replace.ANY)
@Tag(Tags.INTEGRATION)
@Tag(Tags.EXTERNAL_DEPENDENCY)
@Transactional
public class TaskPoolRepositoryManagerTests {
    @Autowired
    TaskPoolRepositoryManager repositoryManager;

    @Autowired
    TaskPoolRepository poolRepository;

    @BeforeEach
    public void beforeAll(@Autowired SubatoTmpDirCleanup tmpDirCleanup) {
        tmpDirCleanup.exec();
    }

    @Test
    public void shouldClonePublicRepo() {
        var pool = poolRepository.findById(1L).get();
        Assertions.assertDoesNotThrow(() -> repositoryManager.cloneRepository(pool));
    }

    /**
     * setting {@link GitConfig#getToken()} for this test is required
     */
    @Test
    public void shouldClonePrivateRepo() {
        var pool = poolRepository.findById(2L).get();
        Assertions.assertDoesNotThrow(() -> repositoryManager.cloneRepository(pool));
    }
}
